@@ -51,6 +51,8 @@ pub enum ScriptContextError {
     MaxRedeemScriptSizeExceeded,
     /// The policy rules of bitcoin core only permit Script size upto 1650 bytes
     MaxScriptSigSizeExceeded,
+    /// Impossible to satisfy the miniscript under the current context
+    ImpossibleSatisfaction,
 }
 
 impl fmt::Display for ScriptContextError {
@@ -89,6 +91,12 @@ impl fmt::Display for ScriptContextError {
                 "At least one satisfaction in Miniscript would be larger than \
                 MAX_SCRIPTSIG_SIZE scriptsig"
             ),
+            ScriptContextError::ImpossibleSatisfaction => {
+                write!(
+                    f,
+                    "Impossible to satisfy Miniscript under the current context"
+                )
+            }
         }
     }
 }
@@ -239,7 +247,7 @@ pub trait ScriptContext:
 /// To be used as P2SH scripts
 /// For creation of Bare scriptpubkeys, construct the Miniscript
 /// under `Bare` ScriptContext
-#[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd)]
+#[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub enum Legacy {}
 
 impl ScriptContext for Legacy {
@@ -293,8 +301,8 @@ impl ScriptContext for Legacy {
         // on P2SH size, it is not possible to reach the 1000 elements limit and hence
         // we do not check it.
         match ms.max_satisfaction_size() {
-            None => Err(ScriptContextError::MaxScriptSigSizeExceeded),
-            Some(size) if size > MAX_SCRIPTSIG_SIZE => {
+            Err(_e) => Err(ScriptContextError::ImpossibleSatisfaction),
+            Ok(size) if size > MAX_SCRIPTSIG_SIZE => {
                 Err(ScriptContextError::MaxScriptSigSizeExceeded)
             }
             _ => Ok(()),
@@ -310,7 +318,7 @@ impl ScriptContext for Legacy {
 }
 
 /// Segwitv0 ScriptContext
-#[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd)]
+#[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub enum Segwitv0 {}
 
 impl ScriptContext for Segwitv0 {
@@ -375,8 +383,9 @@ impl ScriptContext for Segwitv0 {
         // other Segwitv0 defined programs all require (much) less than 100 elements.
         // The witness script item is accounted for in max_satisfaction_witness_elements().
         match ms.max_satisfaction_witness_elements() {
-            None => Err(ScriptContextError::MaxWitnessItemssExceeded),
-            Some(max_witness_items) if max_witness_items > MAX_STANDARD_P2WSH_STACK_ITEMS => {
+            // No possible satisfactions
+            Err(_e) => Err(ScriptContextError::ImpossibleSatisfaction),
+            Ok(max_witness_items) if max_witness_items > MAX_STANDARD_P2WSH_STACK_ITEMS => {
                 Err(ScriptContextError::MaxWitnessItemssExceeded)
             }
             _ => Ok(()),
@@ -395,10 +404,10 @@ impl ScriptContext for Segwitv0 {
 /// To be used as raw script pubkeys
 /// In general, it is not recommended to use Bare descriptors
 /// as they as strongly limited by standardness policies.
-#[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd)]
-pub enum Bare {}
+#[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
+pub enum BareCtx {}
 
-impl ScriptContext for Bare {
+impl ScriptContext for BareCtx {
     fn check_terminal_non_malleable<Pk: MiniscriptKey, Ctx: ScriptContext>(
         _frag: &Terminal<Pk, Ctx>,
     ) -> Result<(), ScriptContextError> {
@@ -457,7 +466,7 @@ impl ScriptContext for Bare {
 /// Used by the "satisified constraints" iterator, which is intended to read
 /// scripts off of the blockchain without doing any sanity checks on them.
 /// This context should not be used unless you know what you are doing.
-#[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd)]
+#[derive(Debug, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub enum NoChecks {}
 impl ScriptContext for NoChecks {
     fn check_terminal_non_malleable<Pk: MiniscriptKey, Ctx: ScriptContext>(
@@ -499,12 +508,12 @@ impl ScriptContext for NoChecks {
 
 /// Private Mod to prevent downstream from implementing this public trait
 mod private {
-    use super::{Bare, Legacy, NoChecks, Segwitv0};
+    use super::{BareCtx, Legacy, NoChecks, Segwitv0};
 
     pub trait Sealed {}
 
     // Implement for those same types, but no others.
-    impl Sealed for Bare {}
+    impl Sealed for BareCtx {}
     impl Sealed for Legacy {}
     impl Sealed for Segwitv0 {}
     impl Sealed for NoChecks {}
