@@ -61,6 +61,7 @@ enum NonTerm {
     EndIfElse,
 }
 /// All AST elements
+#[allow(broken_intra_doc_links)]
 #[derive(Clone, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub enum Terminal<Pk: MiniscriptKey, Ctx: ScriptContext> {
     /// `1`
@@ -154,16 +155,16 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> TerminalStack<Pk, Ctx> {
 
     ///reduce, type check and push a 0-arg node
     fn reduce0(&mut self, ms: Terminal<Pk, Ctx>) -> Result<(), Error> {
-        Ctx::check_frag_validity(&ms)?;
-
         let ty = Type::type_check(&ms, return_none)?;
         let ext = ExtData::type_check(&ms, return_none)?;
-        self.0.push(Miniscript {
+        let ms = Miniscript {
             node: ms,
             ty: ty,
             ext: ext,
             phantom: PhantomData,
-        });
+        };
+        Ctx::check_global_validity(&ms)?;
+        self.0.push(ms);
         Ok(())
     }
 
@@ -174,16 +175,17 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> TerminalStack<Pk, Ctx> {
     {
         let top = self.pop().unwrap();
         let wrapped_ms = wrap(Arc::new(top));
-        Ctx::check_frag_validity(&wrapped_ms)?;
 
         let ty = Type::type_check(&wrapped_ms, return_none)?;
         let ext = ExtData::type_check(&wrapped_ms, return_none)?;
-        self.0.push(Miniscript {
+        let ms = Miniscript {
             node: wrapped_ms,
             ty: ty,
             ext: ext,
             phantom: PhantomData,
-        });
+        };
+        Ctx::check_global_validity(&ms)?;
+        self.0.push(ms);
         Ok(())
     }
 
@@ -196,16 +198,17 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> TerminalStack<Pk, Ctx> {
         let right = self.pop().unwrap();
 
         let wrapped_ms = wrap(Arc::new(left), Arc::new(right));
-        Ctx::check_frag_validity(&wrapped_ms)?;
 
         let ty = Type::type_check(&wrapped_ms, return_none)?;
         let ext = ExtData::type_check(&wrapped_ms, return_none)?;
-        self.0.push(Miniscript {
+        let ms = Miniscript {
             node: wrapped_ms,
             ty: ty,
             ext: ext,
             phantom: PhantomData,
-        });
+        };
+        Ctx::check_global_validity(&ms)?;
+        self.0.push(ms);
         Ok(())
     }
 }
@@ -238,17 +241,45 @@ pub fn parse<Ctx: ScriptContext>(
                         tokens,
                         Tk::Equal => match_token!(
                             tokens,
-                            Tk::Hash20(hash), Tk::Hash160, Tk::Dup => {
-                                term.reduce0(Terminal::PkH(
-                                    hash160::Hash::from_inner(hash)
-                                ))?
-                            },
-                            Tk::Hash32(hash), Tk::Sha256, Tk::Verify, Tk::Equal, Tk::Num(32), Tk::Size => {
-                                non_term.push(NonTerm::Verify);
-                                term.reduce0(Terminal::Sha256(
-                                    sha256::Hash::from_inner(hash)
-                                ))?
-                            },
+                            Tk::Hash20(hash) => match_token!(
+                                tokens,
+                                Tk::Hash160 => match_token!(
+                                    tokens,
+                                    Tk::Dup => {
+                                        term.reduce0(Terminal::PkH(
+                                            hash160::Hash::from_inner(hash)
+                                        ))?
+                                    },
+                                    Tk::Verify, Tk::Equal, Tk::Num(32), Tk::Size => {
+                                        non_term.push(NonTerm::Verify);
+                                        term.reduce0(Terminal::Hash160(
+                                            hash160::Hash::from_inner(hash)
+                                        ))?
+                                    },
+                                ),
+                                Tk::Ripemd160, Tk::Verify, Tk::Equal, Tk::Num(32), Tk::Size => {
+                                    non_term.push(NonTerm::Verify);
+                                    term.reduce0(Terminal::Ripemd160(
+                                        ripemd160::Hash::from_inner(hash)
+                                    ))?
+                                },
+                            ),
+                            // Tk::Hash20(hash),
+                            Tk::Hash32(hash) => match_token!(
+                                tokens,
+                                Tk::Sha256, Tk::Verify, Tk::Equal, Tk::Num(32), Tk::Size => {
+                                    non_term.push(NonTerm::Verify);
+                                    term.reduce0(Terminal::Sha256(
+                                        sha256::Hash::from_inner(hash)
+                                    ))?
+                                },
+                                Tk::Hash256, Tk::Verify, Tk::Equal, Tk::Num(32), Tk::Size => {
+                                    non_term.push(NonTerm::Verify);
+                                    term.reduce0(Terminal::Hash256(
+                                        sha256d::Hash::from_inner(hash)
+                                    ))?
+                                },
+                            ),
                         ),
                         x => {
                             tokens.un_next(x);
