@@ -30,7 +30,7 @@ use std::{fmt, str};
 use bitcoin;
 use bitcoin::blockdata::script;
 
-pub use self::context::{Bare, Legacy, Segwitv0};
+pub use self::context::{BareCtx, Legacy, Segwitv0};
 
 pub mod analyzable;
 pub mod astelem;
@@ -52,7 +52,7 @@ use miniscript::types::Type;
 use std::cmp;
 use std::sync::Arc;
 use MiniscriptKey;
-use {expression, Error, ToPublicKey};
+use {expression, Error, ForEach, ForEachKey, ToPublicKey, TranslatePk};
 
 /// Top-level script AST type
 #[derive(Clone, Hash)]
@@ -166,7 +166,7 @@ impl<Ctx: ScriptContext> Miniscript<bitcoin::PublicKey, Ctx> {
 
     /// Attempt to parse a Script into Miniscript representation.
     /// This function will fail parsing for scripts that do not clear
-    /// the [fn.analyzable.sanity_check] checks. Use [fn.parse_insane] to
+    /// the [Miniscript::sanity_check] checks. Use [Miniscript::parse_insane] to
     /// parse such scripts.
     pub fn parse(script: &script::Script) -> Result<Miniscript<bitcoin::PublicKey, Ctx>, Error> {
         let ms = Self::parse_insane(script)?;
@@ -181,13 +181,11 @@ where
     Ctx: ScriptContext,
 {
     /// Encode as a Bitcoin script
-    pub fn encode<ToPkCtx: Copy>(&self, to_pk_ctx: ToPkCtx) -> script::Script
+    pub fn encode(&self) -> script::Script
     where
-        Pk: ToPublicKey<ToPkCtx>,
+        Pk: ToPublicKey,
     {
-        self.node
-            .encode(script::Builder::new(), to_pk_ctx)
-            .into_script()
+        self.node.encode(script::Builder::new()).into_script()
     }
 
     /// Size, in bytes of the script-pubkey. If this Miniscript is used outside
@@ -197,11 +195,8 @@ where
     /// In general, it is not recommended to use this function directly, but
     /// to instead call the corresponding function on a `Descriptor`, which
     /// will handle the segwit/non-segwit technicalities for you.
-    pub fn script_size<ToPkCtx: Copy>(&self, to_pk_ctx: ToPkCtx) -> usize
-    where
-        Pk: ToPublicKey<ToPkCtx>,
-    {
-        self.node.script_size(to_pk_ctx)
+    pub fn script_size(&self) -> usize {
+        self.node.script_size()
     }
 }
 
@@ -211,11 +206,13 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Miniscript<Pk, Ctx> {
     /// the weight of the `VarInt` that specifies this number in a serialized
     /// transaction.
     ///
-    /// This function may return None on malformed `Miniscript` objects which do
-    /// not correspond to semantically sane Scripts. (Such scripts should be
-    /// rejected at parse time. Any exceptions are bugs.)
-    pub fn max_satisfaction_witness_elements(&self) -> Option<usize> {
-        self.ext.stack_elem_count_sat.map(|x| x + 1)
+    /// This function may returns Error when the Miniscript is
+    /// impossible to satisfy
+    pub fn max_satisfaction_witness_elements(&self) -> Result<usize, Error> {
+        self.ext
+            .stack_elem_count_sat
+            .map(|x| x + 1)
+            .ok_or(Error::ImpossibleSatisfaction)
     }
 
     /// Maximum size, in bytes, of a satisfying witness. For Segwit outputs
@@ -230,16 +227,52 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Miniscript<Pk, Ctx> {
     /// All signatures are assumed to be 73 bytes in size, including the
     /// length prefix (segwit) or push opcode (pre-segwit) and sighash
     /// postfix.
-    pub fn max_satisfaction_size(&self) -> Option<usize> {
-        Ctx::max_satisfaction_size(self)
+    pub fn max_satisfaction_size(&self) -> Result<usize, Error> {
+        Ctx::max_satisfaction_size(self).ok_or(Error::ImpossibleSatisfaction)
+    }
+}
+
+impl<Pk: MiniscriptKey, Ctx: ScriptContext> ForEachKey<Pk> for Miniscript<Pk, Ctx> {
+    fn for_each_key<'a, F: FnMut(ForEach<'a, Pk>) -> bool>(&'a self, mut pred: F) -> bool
+    where
+        Pk: 'a,
+        Pk::Hash: 'a,
+    {
+        self.real_for_each_key(&mut pred)
+    }
+}
+
+impl<Pk: MiniscriptKey, Q: MiniscriptKey, Ctx: ScriptContext> TranslatePk<Pk, Q>
+    for Miniscript<Pk, Ctx>
+{
+    type Output = Miniscript<Q, Ctx>;
+
+    /// This will panic if translatefpk returns an uncompressed key when
+    /// converting to a Segwit descriptor. To prevent this panic, ensure
+    /// translatefpk returns an error in this case instead.
+    fn translate_pk<FPk, FPkh, FuncError>(
+        &self,
+        mut translatefpk: FPk,
+        mut translatefpkh: FPkh,
+    ) -> Result<Self::Output, FuncError>
+    where
+        FPk: FnMut(&Pk) -> Result<Q, FuncError>,
+        FPkh: FnMut(&Pk::Hash) -> Result<Q::Hash, FuncError>,
+    {
+        self.real_translate_pk(&mut translatefpk, &mut translatefpkh)
     }
 }
 
 impl<Pk: MiniscriptKey, Ctx: ScriptContext> Miniscript<Pk, Ctx> {
-    /// This will panic if translatefpk returns an uncompressed key when
-    /// converting to a Segwit descriptor. To prevent this panic, ensure
-    /// translatefpk returns an error in this case instead.
-    pub fn translate_pk<FPk, FPkh, Q, FuncError>(
+    fn real_for_each_key<'a, F: FnMut(ForEach<'a, Pk>) -> bool>(&'a self, pred: &mut F) -> bool
+    where
+        Pk: 'a,
+        Pk::Hash: 'a,
+    {
+        self.node.real_for_each_key(pred)
+    }
+
+    fn real_translate_pk<FPk, FPkh, Q, FuncError>(
         &self,
         translatefpk: &mut FPk,
         translatefpkh: &mut FPkh,
@@ -249,7 +282,7 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Miniscript<Pk, Ctx> {
         FPkh: FnMut(&Pk::Hash) -> Result<Q::Hash, FuncError>,
         Q: MiniscriptKey,
     {
-        let inner = self.node.translate_pk(translatefpk, translatefpkh)?;
+        let inner = self.node.real_translate_pk(translatefpk, translatefpkh)?;
         let ms = Miniscript {
             //directly copying the type and ext is safe because translating public
             //key should not change any properties
@@ -270,6 +303,8 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Miniscript<Pk, Ctx> {
     /// accept sane scripts.
     pub fn from_str_insane(s: &str) -> Result<Miniscript<Pk, Ctx>, Error>
     where
+        Pk: str::FromStr,
+        Pk::Hash: str::FromStr,
         <Pk as str::FromStr>::Err: ToString,
         <<Pk as MiniscriptKey>::Hash as str::FromStr>::Err: ToString,
     {
@@ -293,17 +328,11 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Miniscript<Pk, Ctx> {
 impl<Pk: MiniscriptKey, Ctx: ScriptContext> Miniscript<Pk, Ctx> {
     /// Attempt to produce non-malleable satisfying witness for the
     /// witness script represented by the parse tree
-    pub fn satisfy<ToPkCtx: Copy, S: satisfy::Satisfier<ToPkCtx, Pk>>(
-        &self,
-        satisfier: S,
-        to_pk_ctx: ToPkCtx,
-    ) -> Result<Vec<Vec<u8>>, Error>
+    pub fn satisfy<S: satisfy::Satisfier<Pk>>(&self, satisfier: S) -> Result<Vec<Vec<u8>>, Error>
     where
-        Pk: ToPublicKey<ToPkCtx>,
+        Pk: ToPublicKey,
     {
-        match satisfy::Satisfaction::satisfy(&self.node, &satisfier, self.ty.mall.safe, to_pk_ctx)
-            .stack
-        {
+        match satisfy::Satisfaction::satisfy(&self.node, &satisfier, self.ty.mall.safe).stack {
             satisfy::Witness::Stack(stack) => {
                 Ctx::check_witness::<Pk, Ctx>(&stack)?;
                 Ok(stack)
@@ -316,22 +345,14 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Miniscript<Pk, Ctx> {
 
     /// Attempt to produce a malleable satisfying witness for the
     /// witness script represented by the parse tree
-    pub fn satisfy_malleable<ToPkCtx: Copy, S: satisfy::Satisfier<ToPkCtx, Pk>>(
+    pub fn satisfy_malleable<S: satisfy::Satisfier<Pk>>(
         &self,
         satisfier: S,
-        to_pk_ctx: ToPkCtx,
     ) -> Result<Vec<Vec<u8>>, Error>
     where
-        Pk: ToPublicKey<ToPkCtx>,
+        Pk: ToPublicKey,
     {
-        match satisfy::Satisfaction::satisfy_mall(
-            &self.node,
-            &satisfier,
-            self.ty.mall.safe,
-            to_pk_ctx,
-        )
-        .stack
-        {
+        match satisfy::Satisfaction::satisfy_mall(&self.node, &satisfier, self.ty.mall.safe).stack {
             satisfy::Witness::Stack(stack) => {
                 Ctx::check_witness::<Pk, Ctx>(&stack)?;
                 Ok(stack)
@@ -345,7 +366,8 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Miniscript<Pk, Ctx> {
 
 impl<Pk, Ctx> expression::FromTree for Arc<Miniscript<Pk, Ctx>>
 where
-    Pk: MiniscriptKey,
+    Pk: MiniscriptKey + str::FromStr,
+    Pk::Hash: str::FromStr,
     Ctx: ScriptContext,
     <Pk as str::FromStr>::Err: ToString,
     <<Pk as MiniscriptKey>::Hash as str::FromStr>::Err: ToString,
@@ -357,7 +379,8 @@ where
 
 impl<Pk, Ctx> expression::FromTree for Miniscript<Pk, Ctx>
 where
-    Pk: MiniscriptKey,
+    Pk: MiniscriptKey + str::FromStr,
+    Pk::Hash: str::FromStr,
     Ctx: ScriptContext,
     <Pk as str::FromStr>::Err: ToString,
     <<Pk as MiniscriptKey>::Hash as str::FromStr>::Err: ToString,
@@ -375,18 +398,19 @@ where
     }
 }
 
+/// Parse a Miniscript from string and perform sanity checks
+/// See [Miniscript::from_str_insane] to parse scripts from string that
+/// do not clear the [Miniscript::sanity_check] checks.
 impl<Pk, Ctx> str::FromStr for Miniscript<Pk, Ctx>
 where
-    Pk: MiniscriptKey,
+    Pk: MiniscriptKey + str::FromStr,
+    Pk::Hash: str::FromStr,
     Ctx: ScriptContext,
     <Pk as str::FromStr>::Err: ToString,
     <<Pk as MiniscriptKey>::Hash as str::FromStr>::Err: ToString,
 {
     type Err = Error;
 
-    /// Parse a Miniscript from string and perform sanity checks
-    /// See [fn.from_str_insane] to parse scripts from string that
-    /// do not clear the [fn.analyzable.sanity_check] checks.
     fn from_str(s: &str) -> Result<Miniscript<Pk, Ctx>, Error> {
         let ms = Self::from_str_insane(s)?;
         ms.sanity_check()?;
@@ -405,16 +429,13 @@ mod tests {
     use miniscript::Terminal;
     use policy::Liftable;
     use std::marker::PhantomData;
-    use DummyKey;
-    use DummyKeyHash;
+    use {DummyKey, DummyKeyHash, MiniscriptKey, TranslatePk, TranslatePk1};
 
     use bitcoin::hashes::{hash160, sha256, Hash};
     use bitcoin::{self, secp256k1};
     use std::str;
     use std::str::FromStr;
     use std::sync::Arc;
-    use MiniscriptKey;
-    use NullCtx;
 
     type Segwitv0Script = Miniscript<bitcoin::PublicKey, Segwitv0>;
 
@@ -444,7 +465,8 @@ mod tests {
         expected_debug: Str1,
         expected_display: Str2,
     ) where
-        Pk: MiniscriptKey,
+        Pk: MiniscriptKey + str::FromStr,
+        Pk::Hash: str::FromStr,
         Ctx: ScriptContext,
         <Pk as str::FromStr>::Err: ToString,
         <<Pk as MiniscriptKey>::Hash as str::FromStr>::Err: ToString,
@@ -463,15 +485,17 @@ mod tests {
         let roundtrip = Miniscript::from_str(&display).expect("parse string serialization");
         assert_eq!(roundtrip, script);
 
-        let translated: Result<_, ()> =
-            script.translate_pk(&mut |k| Ok(k.clone()), &mut |h| Ok(h.clone()));
-        assert_eq!(translated, Ok(script));
+        let translated = script.translate_pk_infallible(Pk::clone, Pk::Hash::clone);
+        assert_eq!(translated, script);
+
+        let translated = script.translate_pk1_infallible(Pk::clone);
+        assert_eq!(translated, script);
     }
 
     fn script_rtt<Str1: Into<Option<&'static str>>>(script: Segwitv0Script, expected_hex: Str1) {
         assert_eq!(script.ty.corr.base, types::Base::B);
-        let bitcoin_script = script.encode(NullCtx);
-        assert_eq!(bitcoin_script.len(), script.script_size(NullCtx));
+        let bitcoin_script = script.encode();
+        assert_eq!(bitcoin_script.len(), script.script_size());
         if let Some(expected) = expected_hex.into() {
             assert_eq!(format!("{:x}", bitcoin_script), expected);
         }
@@ -482,8 +506,8 @@ mod tests {
 
     fn roundtrip(tree: &Segwitv0Script, s: &str) {
         assert_eq!(tree.ty.corr.base, types::Base::B);
-        let ser = tree.encode(NullCtx);
-        assert_eq!(ser.len(), tree.script_size(NullCtx));
+        let ser = tree.encode();
+        assert_eq!(ser.len(), tree.script_size());
         assert_eq!(ser.to_string(), s);
         let deser = Segwitv0Script::parse_insane(&ser).expect("deserialize result of serialize");
         assert_eq!(*tree, deser);
@@ -501,7 +525,7 @@ mod tests {
         let ms: Result<Segwitv0Script, _> = Miniscript::from_str_insane(ms);
         match (ms, valid) {
             (Ok(ms), true) => {
-                assert_eq!(format!("{:x}", ms.encode(NullCtx)), expected_hex);
+                assert_eq!(format!("{:x}", ms.encode()), expected_hex);
                 assert_eq!(ms.ty.mall.non_malleable, non_mal);
                 assert_eq!(ms.ty.mall.safe, need_sig);
                 assert_eq!(ms.ext.ops_count_sat.unwrap(), ops);
@@ -650,19 +674,19 @@ mod tests {
     fn verify_parse() {
         let ms = "and_v(v:hash160(20195b5a3d650c17f0f29f91c33f8f6335193d07),or_d(sha256(96de8fc8c256fa1e1556d41af431cace7dca68707c78dd88c3acab8b17164c47),older(16)))";
         let ms: Segwitv0Script = Miniscript::from_str_insane(ms).unwrap();
-        assert_eq!(ms, Miniscript::parse_insane(&ms.encode(NullCtx)).unwrap());
+        assert_eq!(ms, Miniscript::parse_insane(&ms.encode()).unwrap());
 
         let ms = "and_v(v:sha256(96de8fc8c256fa1e1556d41af431cace7dca68707c78dd88c3acab8b17164c47),or_d(sha256(96de8fc8c256fa1e1556d41af431cace7dca68707c78dd88c3acab8b17164c47),older(16)))";
         let ms: Segwitv0Script = Miniscript::from_str_insane(ms).unwrap();
-        assert_eq!(ms, Miniscript::parse_insane(&ms.encode(NullCtx)).unwrap());
+        assert_eq!(ms, Miniscript::parse_insane(&ms.encode()).unwrap());
 
         let ms = "and_v(v:ripemd160(20195b5a3d650c17f0f29f91c33f8f6335193d07),or_d(sha256(96de8fc8c256fa1e1556d41af431cace7dca68707c78dd88c3acab8b17164c47),older(16)))";
         let ms: Segwitv0Script = Miniscript::from_str_insane(ms).unwrap();
-        assert_eq!(ms, Miniscript::parse_insane(&ms.encode(NullCtx)).unwrap());
+        assert_eq!(ms, Miniscript::parse_insane(&ms.encode()).unwrap());
 
         let ms = "and_v(v:hash256(96de8fc8c256fa1e1556d41af431cace7dca68707c78dd88c3acab8b17164c47),or_d(sha256(96de8fc8c256fa1e1556d41af431cace7dca68707c78dd88c3acab8b17164c47),older(16)))";
         let ms: Segwitv0Script = Miniscript::from_str_insane(ms).unwrap();
-        assert_eq!(ms, Miniscript::parse_insane(&ms.encode(NullCtx)).unwrap());
+        assert_eq!(ms, Miniscript::parse_insane(&ms.encode()).unwrap());
     }
 
     #[test]
