@@ -1169,7 +1169,6 @@ mod tests {
     use policy::Liftable;
     use script_num_size;
     use BitcoinSig;
-    use NullCtx;
 
     type SPolicy = Concrete<String>;
     type BPolicy = Concrete<bitcoin::PublicKey>;
@@ -1271,17 +1270,18 @@ mod tests {
             compilation.ms.lift().unwrap().sorted()
         );
 
-        let policy = SPolicy::from_str(
-                "and(and(and(or(127@thresh(2,pk(A),pk(B),thresh(2,or(127@pk(A),1@pk(B)),after(100),or(and(pk(C),after(200)),and(pk(D),sha256(66687aadf862bd776c8fc18b8e9f8e20089714856ee233b3902a591d0d5f2925))),pk(E))),1@pk(F)),sha256(66687aadf862bd776c8fc18b8e9f8e20089714856ee233b3902a591d0d5f2925)),or(127@pk(G),1@after(300))),or(127@after(400),pk(H)))"
-            ).expect("parsing");
-        let compilation: DummySegwitAstElemExt =
-            best_t(&mut BTreeMap::new(), &policy, 1.0, None).unwrap();
+        // This test is commented as it now exceeds the resource limits. Because of the backport-fix for `u` property in `d`, compilation exceeds the resource limits.
+        // let policy = SPolicy::from_str(
+        //         "and(and(or(127@thresh(2,pk(A),pk(B),thresh(2,or(127@pk(A),1@pk(B)),after(100),or(and(pk(C),after(200)),and(pk(D),sha256(66687aadf862bd776c8fc18b8e9f8e20089714856ee233b3902a591d0d5f2925))),pk(E))),1@pk(F)),sha256(66687aadf862bd776c8fc18b8e9f8e20089714856ee233b3902a591d0d5f2925)),or(127@pk(G),1@after(300)))"
+        //     ).expect("parsing");
+        // let compilation: DummySegwitAstElemExt =
+        //     best_t(&mut BTreeMap::new(), &policy, 1.0, None).unwrap();
 
-        assert_eq!(compilation.cost_1d(1.0, None), 438.0 + 299.4003295898438);
-        assert_eq!(
-            policy.lift().unwrap().sorted(),
-            compilation.ms.lift().unwrap().sorted()
-        );
+        // assert_eq!(compilation.cost_1d(1.0, None), 437.0 + 299.4003295898438);
+        // assert_eq!(
+        //     policy.lift().unwrap().sorted(),
+        //     compilation.ms.lift().unwrap().sorted()
+        // );
     }
 
     #[test]
@@ -1292,7 +1292,7 @@ mod tests {
         let policy: BPolicy = Concrete::Key(keys[0].clone());
         let ms: SegwitMiniScript = policy.compile().unwrap();
         assert_eq!(
-            ms.encode(NullCtx),
+            ms.encode(),
             script::Builder::new()
                 .push_key(&keys[0])
                 .push_opcode(opcodes::all::OP_CHECKSIG)
@@ -1308,7 +1308,7 @@ mod tests {
         );
         let ms: SegwitMiniScript = policy.compile().unwrap();
         assert_eq!(
-            ms.encode(NullCtx),
+            ms.encode(),
             script::Builder::new()
                 .push_opcode(opcodes::all::OP_PUSHNUM_2)
                 .push_key(&keys[5])
@@ -1380,19 +1380,14 @@ mod tests {
             right_sat.insert(keys[i].to_pubkeyhash(), (keys[i], bitcoinsig));
         }
 
-        assert!(ms.satisfy(no_sat, NullCtx).is_err());
-        assert!(ms.satisfy(&left_sat, NullCtx).is_ok());
-        assert!(ms
-            .satisfy((&right_sat, satisfy::Older(10001)), NullCtx)
-            .is_ok());
+        assert!(ms.satisfy(no_sat).is_err());
+        assert!(ms.satisfy(&left_sat).is_ok());
+        assert!(ms.satisfy((&right_sat, satisfy::Older(10001))).is_ok());
         //timelock not met
-        assert!(ms
-            .satisfy((&right_sat, satisfy::Older(9999)), NullCtx)
-            .is_err());
+        assert!(ms.satisfy((&right_sat, satisfy::Older(9999))).is_err());
 
         assert_eq!(
-            ms.satisfy((left_sat, satisfy::Older(9999)), NullCtx)
-                .unwrap(),
+            ms.satisfy((left_sat, satisfy::Older(9999))).unwrap(),
             vec![
                 // sat for left branch
                 vec![],
@@ -1403,8 +1398,7 @@ mod tests {
         );
 
         assert_eq!(
-            ms.satisfy((right_sat, satisfy::Older(10000)), NullCtx)
-                .unwrap(),
+            ms.satisfy((right_sat, satisfy::Older(10000))).unwrap(),
             vec![
                 // sat for right branch
                 vec![],
@@ -1451,14 +1445,11 @@ mod tests {
             let big_thresh_ms: SegwitMiniScript = big_thresh.compile().unwrap();
             if *k == 21 {
                 // N * (PUSH + pubkey + CHECKSIGVERIFY)
-                assert_eq!(
-                    big_thresh_ms.script_size(NullCtx),
-                    keys.len() * (1 + 33 + 1)
-                );
+                assert_eq!(big_thresh_ms.script_size(), keys.len() * (1 + 33 + 1));
             } else {
                 // N * (PUSH + pubkey + CHECKSIG + ADD + SWAP) + N EQUAL
                 assert_eq!(
-                    big_thresh_ms.script_size(NullCtx),
+                    big_thresh_ms.script_size(),
                     keys.len() * (1 + 33 + 3) + script_num_size(*k) + 1 - 2 // minus one SWAP and one ADD
                 );
                 let big_thresh_ms_expected = ms_str!(
@@ -1490,7 +1481,7 @@ mod tests {
             (1, Concrete::Threshold(keys_b.len(), keys_b)),
         ])
         .compile();
-        let script_size = thresh_res.clone().and_then(|m| Ok(m.script_size(NullCtx)));
+        let script_size = thresh_res.clone().and_then(|m| Ok(m.script_size()));
         assert_eq!(
             thresh_res,
             Err(CompilerError::LimitsExceeded),
